/// Importer le module `Training` depuis le fichier externe
use crate::training::Training;

/// Importations des bibliothèques standard et de cursive
use std::rc::Rc;
use cursive::Cursive;
use cursive::{
    views::{Button, Dialog, DummyView, EditView, LinearLayout, SelectView, TextView, NamedView, ResizedView},
    traits::*,
};

/// Fonction publique pour générer le TUI
pub fn run() {
    // Initialisation de Cursive
    let mut siv = cursive::default();

    // Vue des entraînements
    let select = SelectView::<Training>::new()
        .on_submit(on_submit) // Appelé lors de la soumission de la sélection
        .with_name("select") // Nom de la vue
        .fixed_size((10, 5)); // Taille fixe de la vue

    // Buttons pour ajouter et supprimer un entraînement
    let buttons = LinearLayout::vertical()
        .child(Button::new("Add new", add_training)) // Bouton pour ajouter un nouvel entraînement
        .child(Button::new("Delete", delete_training)) // Bouton pour supprimer un entraînement
        .child(DummyView) // Vue fictive pour ajouter de l'espace
        .child(Button::new("Quit", application_close)); // Bouton pour quitter l'application

    // Ajout de la couche de dialogue principale à Cursive
    siv.add_layer(Dialog::around(LinearLayout::horizontal()
        .child(select) // Ajout de la vue de sélection
        .child(DummyView) // Ajout d'espace vide
        .child(buttons)) // Ajout des boutons
        .title("Training session")); // Titre de la couche de dialogue

    // Rajouter les entraînements sauvegarder dans le json
    let trainings = load_from_json("training_session.json");
    match trainings {
        Ok(training_list) => {
            for training in training_list.iter() {
                siv.call_on_name("select", |view: &mut SelectView<Training>| {
                    view.add_item(training.name.clone(), training.clone());
                });
            }
        }
        Err(err) => eprintln!("Error loading from JSON: {}", err),
    }

    // Exécution de la boucle principale de Cursive
    siv.run();
}

/// Fonction interne pour gérer l'ajout d'un nouvel entraînement
///
/// # Arguments
///
/// * `s` - Une référence mutable à Cursive
fn add_training(s: &mut Cursive) {
    fn ok(s: &mut Cursive, training: Training) {
        // Ajout de l'entraînement à la vue de sélection
        s.call_on_name("select", |view: &mut SelectView<Training>| {
            view.add_item(training.name.clone(), training.clone());
        });

        s.pop_layer(); // Fermer la couche de dialogue
    }

    // Ajout d'une couche de dialogue pour l'ajout d'un nouvel entraînement
    s.add_layer(Dialog::around(
        LinearLayout::vertical()
            .with(
                |list| for item in Training::TRAINING_ITEMS.iter() {
                    list.add_child(create_label(item)); // Ajout d'un label pour chaque élément de l'entraînement
                    list.add_child(create_edit_view(item)); // Ajout d'un champ de saisie pour chaque élément
                    list.add_child(DummyView); // Ajout d'espace vide
                })
            .child(DummyView) // Ajout d'espace vide
            .child(LinearLayout::horizontal()
                .child(Button::new(
                    "Ok", |s| { // Bouton "Ok" pour valider l'ajout
                        // Récupération des données saisies par l'utilisateur
                        let name = get_view_content(s, "name");
                        let description = get_view_content(s, "description");
                        // TODO Vérifier format date
                        let date = get_view_content(s, "date");
                        let location = get_view_content(s, "location");
                        let duration = get_view_content(s, "duration").parse::<i32>().unwrap_or(0);
                        let participants_str = get_view_content(s, "participants").to_string();
                        let participants = participants_str.split(",").map(str::trim).collect();

                        // Vérification si le nom est vide
                        if name.is_empty() {
                            s.add_layer(Dialog::info("Name cannot be empty")); // Affichage d'un message d'erreur
                        } else {
                            ok(s, Training::new(&name, &description, &date, &location, duration, participants)); // Appel de la fonction `ok` pour gérer l'ajout
                        }
                    }))
                .child(Button::new("Cancel", |s| {
                    s.pop_layer(); // Annuler l'ajout et fermer la couche de dialogue
                }))))
                    .title("Enter a new training") // Titre de la couche de dialogue
                    .scrollable() // Permettre le défilement si le contenu dépasse
    );
}

/// Récupérer le contenu de la vue
///
/// # Arguments
///
/// * `s` - Une référence mutable à Cursive
/// * `name` - Le nom du champ de saisie
fn get_view_content(s: &mut Cursive, name: &str) -> Rc<String> {
    s.call_on_name(name, |view: &mut EditView| view.get_content()).unwrap()
}

/// Créer un label pour un champ de l'entraînement
///
/// # Arguments
///
/// * `name` - Le nom du champ
fn create_label(name: &str) -> NamedView<TextView> {
    TextView::new(format!("{}:", name)).with_name(format!("{}_label", &name))
}

/// Créer un champ de saisie pour un champ de l'entraînement
///
/// # Arguments
///
/// * `name` - Le nom du champ
fn create_edit_view(name: &str) -> ResizedView<NamedView<EditView>> {
    EditView::new()
        .with_name(name)
        .fixed_width(18)
}

/// Supprimer un entraînement
///
/// # Arguments
///
/// * `s` - Une référence mutable à Cursive
fn delete_training(s: &mut Cursive) {
    let mut select = s.find_name::<SelectView<Training>>("select").unwrap(); // Trouver la vue de sélection
    match select.selected_id() {
        None => s.add_layer(Dialog::info("No train to remove")), // Afficher un message s'il n'y a pas de formation sélectionnée
        Some(focus) => {
            select.remove_item(focus); // Supprimer l'entraînement sélectionnée
        }
    }
}

/// Appelé lorsque l'utilisateur soumet un entraînement
///
/// # Arguments
///
/// * `s` - Une référence mutable à Cursive
/// * `training` - Une référence à la structure Training soumise
fn on_submit(s: &mut Cursive, training: &Training) {
    // Afficher les informations de l'entraînement dans une couche de dialogue
    s.add_layer(Dialog::text(training.to_string())
                    .title(format!("{}'s info", training.name)) // Titre de la couche de dialogue
                    .button("Quit", |s| {
                        s.pop_layer(); // Ajouter un bouton pour fermer la couche de dialogue
                    })
                    .scrollable() // Permettre le défilement si le contenu dépasse
    );
}

/// Ferme l'application après avoir sauvegardé les données de la session d'entraînement
/// dans un fichier json.
///
/// # Arguments
///
/// * `s` - Une référence mutable à Cursive, une bibliothèque de terminal TUI pour Rust.
fn application_close(s: &mut Cursive) {
    let select = s.find_name::<SelectView<Training>>("select").unwrap();
    let trainings = Training::extract_trainings_from_view(&select);
    if let Err(err) = save_to_json(&trainings, "training_session.json") {
        eprintln!("Error saving to JSON: {}", err);
    }
    s.quit();
}


/// Enregistre les données des entraînements au format JSON dans un fichier spécifié.
///
/// # Arguments
///
/// * `trainings` - Une référence au vecteur contenant les données des entraînements à sauvegarder.
/// * `file_path` - Le chemin du fichier de destination pour sauvegarder les données au format JSON.
///
/// # Returns
///
/// Retourne `Result` indiquant le succès de l'enregistrement des données au format JSON
/// ou une erreur de type `std::io::Error`.
fn save_to_json(trainings: &Vec<&Training>, file_path: &str) -> std::io::Result<()> {
    let serialized = serde_json::to_string_pretty(trainings)?;
    std::fs::write(file_path, serialized)?;
    Ok(())
}


/// Charge les données d'entraînement à partir d'un fichier JSON spécifié.
///
/// # Arguments
///
/// * `file_path` - Le chemin du fichier JSON à charger.
///
/// # Returns
///
/// Retourne `Result` contenant un vecteur de `Training` chargé à partir du fichier JSON
/// ou une erreur de type `std::io::Error`.
fn load_from_json(file_path: &str) -> std::io::Result<Vec<Training>> {
    let contents = std::fs::read_to_string(file_path)?;
    let data: Vec<Training> = serde_json::from_str(&contents)?;
    Ok(data)
}
