/// Importe les crates externes `serde` et `serde_json`.
extern crate serde;
extern crate serde_json;

/// Importe la vue `SelectView` du crate `cursive`.
use cursive::views::SelectView;

/// Structure représentant un entraînement
#[derive(Clone, serde::Serialize, serde::Deserialize)]
pub struct Training {
    /// Nom de l'entraînement
    pub name: String,
    /// Description de l'entraînement
    pub description: String,
    /// Date de l'entraînement
    pub date: String,
    /// Lieu de l'entraînement
    pub location: String,
    /// Durée de l'entraînement en heures
    pub duration: i32,
    /// Liste des participants à l'entraînement
    pub participants: Vec<String>,
}

impl Training {
    /// Tableau des éléments de l'entraînement
    pub const TRAINING_ITEMS: [&'static str; 6] = ["name", "description", "date", "location", "duration", "participants"];

    /// Crée une nouvelle instance de la structure Training
    ///
    /// # Arguments
    ///
    /// * `name` - Le nom de l'entraînement
    /// * `description` - La description de l'entraînement
    /// * `date` - La date de l'entraînement
    /// * `location` - Le lieu de l'entraînement
    /// * `duration` - La durée de l'entraînement en heures
    /// * `participants` - Les participants à l'entraînement sous forme de vecteur de chaînes de caractères
    pub fn new(
        name: &str,
        description: &str,
        date: &str,
        location: &str,
        duration: i32,
        participants: Vec<&str>
    ) -> Training {
        Training {
            name: name.to_string(),
            description: description.to_string(),
            date: date.to_string(),
            location: location.to_string(),
            duration,
            participants: participants.iter().map(ToString::to_string).collect(),
        }
    }

    /// Convertit la structure Training en une chaîne de caractères formatée
    pub fn to_string(&self) -> String {
        format!("Name: {}\nDescription: {}\nDate: {}\nLocation: {}\nDuration: {}\nParticipants: {}",
                self.name, self.description, self.date, self.location, self.duration.to_string(), self.participants.join(","))
    }

    /// Extrait la liste des entraînements de la vue `SelectView`.
    ///
    /// # Arguments
    ///
    /// * `select_view` - Une référence à la vue `SelectView` contenant les données d'entraînement.
    ///
    /// # Returns
    ///
    /// Retourne un vecteur contenant des références aux données d'entraînement extraites de la vue.
    pub fn extract_trainings_from_view(select_view: &SelectView<Training>) -> Vec<&Training> {
        let items = select_view.len();
        let mut trainings = Vec::with_capacity(items);

        for i in 0..items {
            if let Some(training) = select_view.get_item(i) {
                trainings.push(training.clone().1);
            }
        }

        trainings
    }
}